#ifndef TMDlib_H
#define TMDlib_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <cstring>
#include <math.h>
#include <vector>
#include <iostream>

using namespace std;

namespace TMDlib {

  double ipow(double,int);
  extern int iipow(int,int);
  // to initialise TMD with name ( and optionally with iset and imode
  void TMDinit(const string name, int irep, int imode  ) ;
  void TMDinit(const string name, int irep ) ;
  void TMDinit(const string name) ;
  
  // to return TMD value at x,xbar,kt,mu as a vector
  vector<double> TMDpdf( double x, double xbar, double kt, double mu) ;
  // to return TMD value at x,xbar,kt,mu as a vector
  void TMDpdf( double x, double xbar, double kt, double mu, vector<double>& xpq) ;
  // to return TMD value at x,xbar,kt,mu for flavor
  void TMDpdf( double x, double xbar, double kt, double mu, double& uval, double& dval, double& sea, double& charm, double& bottom, double& gluon );
  // to return TMD value at x,xbar,kt,mu as a vector
  vector<double> TMDpdf(int kf, double x, double xbar, double kt, double mu) ;
  // to return TMD value at x,xbar,kt,mu as a vector
  void TMDpdf(int kf,  double x, double xbar, double kt, double mu, vector<double>& xpq) ;
  // to return TMD value at x,xbar,kt,mu for flavor
  void TMDpdf(int kf,  double x, double xbar, double kt, double mu, double& uval, double& dval, double& sea, double& charm, double& bottom, double& gluon );
  
  double TMDalphas(double mu);
  
  double TMDgetLam4( );
  int TMDgetNf( );
  int TMDgetOrderAlphaS( );
  int TMDgetOrderPDF( );
  double TMDgetXmin();
  double TMDgetXmax();
  double TMDgetQ2min();
  double TMDgetQ2max();
  int TMDnumberPDF(string name);
// tools

  double Cdhfint(int narg, double arg[500], int nent[500], double ent[500], double table[1000000]);  
  void polint(double xa[], double ya[], int n, double x, double& y, double& dy) ;
  void polin3(double x1a[], double x2a[], double x3a[], double ya[][1][4], int m, int n, int o, double x1, double x2, double x3, double& y, double& dy); 

  
// ccfm stuff
  void ccfmuPDF(int kf, double x, double kt , double p, double& uval, double& dval, double& sea, double& charm, double& bottom, double& gluon ) ;
  void ccfmiPDF(int kf, double x, double p, double& uval, double& dval, double& sea, double& charm, double& bottom, double& gluon ) ;
  

// GBW 
  void gbwuPDFlight(double x, double kt, double& gluon ) ;
  void gbwuPDFcharm(double x, double kt, double& gluon ) ;


// SBRS
  void sbrsPDF(int irep, int imode, double x, double kt, double p, double& uval, double& dval, double& sea, double& charm);

// Kutak Sapeta
  void ksPDF(double x, double kt, double p, double& uval, double& dval, double& sea, double& charm, double& bottom, double& gluon);
  void ksuPDFgrid(int kf, double x, double kt , double p, double& uval, double& dval, double& sea, double& charm, double& bottom, double& gluon ) ;

// M. Echevaria TMDgluon  
  void tmd_ME(int kf, double x, double kt , double p, double& uval, double& dval, double& sea, double& charm, double& bottom, double& gluon ) ;
 
   extern "C" { 
   // Fortran routines
   // remember FORTRAN passes pointer (instead of values) to subroutines
   #define initalphas initalphas_
   void initalphas(int* iord, double* fr2, double* mur , double* asmur, double* mc, double* mb, double* mt); 	
   #define unpol unpol_
   void unpol(int* in , double* scale, double* scale2, double* bc, double* lam1, double* lam2, double* x_ME, double* kt_ME,double* Qf, double* result ); 
  // #define asmur _asmur
    }

     
    
  
}

#endif
